#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <AIS_Shape.hxx>
#include <BRepPrimAPI_MakeBox.hxx>
#include <BRepPrimAPI_MakeCone.hxx>
#include "GeomAPI_Interpolate.hxx"
#include "BRepBuilderAPI_MakeEdge.hxx"
#include <BRepBuilderAPI_MakeWire.hxx>
#include <BRepBuilderAPI_Transform.hxx>
#include <BRepOffsetAPI_ThruSections.hxx>
#include <TopoDS.hxx>

#include <Prs3d_PointAspect.hxx>
#include <Prs3d_LineAspect.hxx>
#include <QBoxLayout>

#include <iostream>
#include <vector>

using namespace std;

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent), ui(new Ui::MainWindow) {
    ui->setupUi(this);
    DocumentCommon* dc = new DocumentCommon ();
    mview = new View (dc->getContext(),this);
    setCentralWidget(mview);;

    connect(ui->Wing, SIGNAL(triggered()), this, SLOT(makeWing()));
    connect(ui->Refresh, SIGNAL(triggered()), this, SLOT(Refresh()));
}


void MainWindow::Refresh(){
	mview->getContext() -> Remove(aisShape);
    mview->getContext() ->CloseAllContexts(false);
    mview->getContext() ->OpenLocalContext(false, false, false);
	mview->getContext() -> Display(aisShape);
}


void MainWindow::makeWing () {

    int sampleLen = 201;

    Handle(TColgp_HArray1OfPnt) samples = new TColgp_HArray1OfPnt(1, sampleLen);
    Handle(TColgp_HArray1OfPnt) samples2 = new TColgp_HArray1OfPnt(1, sampleLen);
    Handle(TColgp_HArray1OfPnt) samples3 = new TColgp_HArray1OfPnt(1, sampleLen);
    Handle(TColgp_HArray1OfPnt) samples4 = new TColgp_HArray1OfPnt(1, sampleLen);

	double GeometryTolerance = 0.000001;

// sampling of NACA0012 points

vector<gp_Pnt> pnt, pnt2, pnt3, pnt4;

pnt.push_back( gp_Pnt( 0, 0, 0) );
pnt2.push_back( gp_Pnt( 0, 0, 0) );
pnt3.push_back( gp_Pnt( 80.3848, 300, 0) );
pnt4.push_back( gp_Pnt( 80.3848, 300, 0) );

pnt.push_back( gp_Pnt( 0.5, 0, 1.1619) );
pnt2.push_back( gp_Pnt( 0.5, 0, -1.1619) );
pnt3.push_back( gp_Pnt( 80.6848, 300, 0.697143) );
pnt4.push_back( gp_Pnt( 80.6848, 300, -0.697143) );

pnt.push_back( gp_Pnt( 1, 0, 1.63623) );
pnt2.push_back( gp_Pnt( 1, 0, -1.63623) );
pnt3.push_back( gp_Pnt( 80.9848, 300, 0.981737) );
pnt4.push_back( gp_Pnt( 80.9848, 300, -0.981737) );

pnt.push_back( gp_Pnt( 1.5, 0, 1.99529) );
pnt2.push_back( gp_Pnt( 1.5, 0, -1.99529) );
pnt3.push_back( gp_Pnt( 81.2848, 300, 1.19717) );
pnt4.push_back( gp_Pnt( 81.2848, 300, -1.19717) );

pnt.push_back( gp_Pnt( 2, 0, 2.29376) );
pnt2.push_back( gp_Pnt( 2, 0, -2.29376) );
pnt3.push_back( gp_Pnt( 81.5848, 300, 1.37626) );
pnt4.push_back( gp_Pnt( 81.5848, 300, -1.37626) );

pnt.push_back( gp_Pnt( 2.5, 0, 2.5529) );
pnt2.push_back( gp_Pnt( 2.5, 0, -2.5529) );
pnt3.push_back( gp_Pnt( 81.8848, 300, 1.53174) );
pnt4.push_back( gp_Pnt( 81.8848, 300, -1.53174) );

pnt.push_back( gp_Pnt( 3, 0, 2.78365) );
pnt2.push_back( gp_Pnt( 3, 0, -2.78365) );
pnt3.push_back( gp_Pnt( 82.1848, 300, 1.67019) );
pnt4.push_back( gp_Pnt( 82.1848, 300, -1.67019) );

pnt.push_back( gp_Pnt( 3.5, 0, 2.99252) );
pnt2.push_back( gp_Pnt( 3.5, 0, -2.99252) );
pnt3.push_back( gp_Pnt( 82.4848, 300, 1.79551) );
pnt4.push_back( gp_Pnt( 82.4848, 300, -1.79551) );

pnt.push_back( gp_Pnt( 4, 0, 3.18377) );
pnt2.push_back( gp_Pnt( 4, 0, -3.18377) );
pnt3.push_back( gp_Pnt( 82.7848, 300, 1.91026) );
pnt4.push_back( gp_Pnt( 82.7848, 300, -1.91026) );

pnt.push_back( gp_Pnt( 4.5, 0, 3.36035) );
pnt2.push_back( gp_Pnt( 4.5, 0, -3.36035) );
pnt3.push_back( gp_Pnt( 83.0848, 300, 2.01621) );
pnt4.push_back( gp_Pnt( 83.0848, 300, -2.01621) );

pnt.push_back( gp_Pnt( 5, 0, 3.52445) );
pnt2.push_back( gp_Pnt( 5, 0, -3.52445) );
pnt3.push_back( gp_Pnt( 83.3848, 300, 2.11467) );
pnt4.push_back( gp_Pnt( 83.3848, 300, -2.11467) );

pnt.push_back( gp_Pnt( 5.5, 0, 3.67769) );
pnt2.push_back( gp_Pnt( 5.5, 0, -3.67769) );
pnt3.push_back( gp_Pnt( 83.6848, 300, 2.20662) );
pnt4.push_back( gp_Pnt( 83.6848, 300, -2.20662) );

pnt.push_back( gp_Pnt( 6, 0, 3.82136) );
pnt2.push_back( gp_Pnt( 6, 0, -3.82136) );
pnt3.push_back( gp_Pnt( 83.9848, 300, 2.29282) );
pnt4.push_back( gp_Pnt( 83.9848, 300, -2.29282) );

pnt.push_back( gp_Pnt( 6.5, 0, 3.95648) );
pnt2.push_back( gp_Pnt( 6.5, 0, -3.95648) );
pnt3.push_back( gp_Pnt( 84.2848, 300, 2.37389) );
pnt4.push_back( gp_Pnt( 84.2848, 300, -2.37389) );

pnt.push_back( gp_Pnt( 7, 0, 4.08387) );
pnt2.push_back( gp_Pnt( 7, 0, -4.08387) );
pnt3.push_back( gp_Pnt( 84.5848, 300, 2.45032) );
pnt4.push_back( gp_Pnt( 84.5848, 300, -2.45032) );

pnt.push_back( gp_Pnt( 7.5, 0, 4.20422) );
pnt2.push_back( gp_Pnt( 7.5, 0, -4.20422) );
pnt3.push_back( gp_Pnt( 84.8848, 300, 2.52253) );
pnt4.push_back( gp_Pnt( 84.8848, 300, -2.52253) );

pnt.push_back( gp_Pnt( 8, 0, 4.31811) );
pnt2.push_back( gp_Pnt( 8, 0, -4.31811) );
pnt3.push_back( gp_Pnt( 85.1848, 300, 2.59087) );
pnt4.push_back( gp_Pnt( 85.1848, 300, -2.59087) );

pnt.push_back( gp_Pnt( 8.5, 0, 4.42604) );
pnt2.push_back( gp_Pnt( 8.5, 0, -4.42604) );
pnt3.push_back( gp_Pnt( 85.4848, 300, 2.65562) );
pnt4.push_back( gp_Pnt( 85.4848, 300, -2.65562) );

pnt.push_back( gp_Pnt( 9, 0, 4.52841) );
pnt2.push_back( gp_Pnt( 9, 0, -4.52841) );
pnt3.push_back( gp_Pnt( 85.7848, 300, 2.71705) );
pnt4.push_back( gp_Pnt( 85.7848, 300, -2.71705) );

pnt.push_back( gp_Pnt( 9.5, 0, 4.6256) );
pnt2.push_back( gp_Pnt( 9.5, 0, -4.6256) );
pnt3.push_back( gp_Pnt( 86.0848, 300, 2.77536) );
pnt4.push_back( gp_Pnt( 86.0848, 300, -2.77536) );

pnt.push_back( gp_Pnt( 10, 0, 4.71793) );
pnt2.push_back( gp_Pnt( 10, 0, -4.71793) );
pnt3.push_back( gp_Pnt( 86.3848, 300, 2.83076) );
pnt4.push_back( gp_Pnt( 86.3848, 300, -2.83076) );

pnt.push_back( gp_Pnt( 10.5, 0, 4.80567) );
pnt2.push_back( gp_Pnt( 10.5, 0, -4.80567) );
pnt3.push_back( gp_Pnt( 86.6848, 300, 2.8834) );
pnt4.push_back( gp_Pnt( 86.6848, 300, -2.8834) );

pnt.push_back( gp_Pnt( 11, 0, 4.88909) );
pnt2.push_back( gp_Pnt( 11, 0, -4.88909) );
pnt3.push_back( gp_Pnt( 86.9848, 300, 2.93345) );
pnt4.push_back( gp_Pnt( 86.9848, 300, -2.93345) );

pnt.push_back( gp_Pnt( 11.5, 0, 4.96839) );
pnt2.push_back( gp_Pnt( 11.5, 0, -4.96839) );
pnt3.push_back( gp_Pnt( 87.2848, 300, 2.98104) );
pnt4.push_back( gp_Pnt( 87.2848, 300, -2.98104) );

pnt.push_back( gp_Pnt( 12, 0, 5.0438) );
pnt2.push_back( gp_Pnt( 12, 0, -5.0438) );
pnt3.push_back( gp_Pnt( 87.5848, 300, 3.02628) );
pnt4.push_back( gp_Pnt( 87.5848, 300, -3.02628) );

pnt.push_back( gp_Pnt( 12.5, 0, 5.11548) );
pnt2.push_back( gp_Pnt( 12.5, 0, -5.11548) );
pnt3.push_back( gp_Pnt( 87.8848, 300, 3.06929) );
pnt4.push_back( gp_Pnt( 87.8848, 300, -3.06929) );

pnt.push_back( gp_Pnt( 13, 0, 5.18361) );
pnt2.push_back( gp_Pnt( 13, 0, -5.18361) );
pnt3.push_back( gp_Pnt( 88.1848, 300, 3.11017) );
pnt4.push_back( gp_Pnt( 88.1848, 300, -3.11017) );

pnt.push_back( gp_Pnt( 13.5, 0, 5.24834) );
pnt2.push_back( gp_Pnt( 13.5, 0, -5.24834) );
pnt3.push_back( gp_Pnt( 88.4848, 300, 3.14901) );
pnt4.push_back( gp_Pnt( 88.4848, 300, -3.14901) );

pnt.push_back( gp_Pnt( 14, 0, 5.30981) );
pnt2.push_back( gp_Pnt( 14, 0, -5.30981) );
pnt3.push_back( gp_Pnt( 88.7848, 300, 3.18588) );
pnt4.push_back( gp_Pnt( 88.7848, 300, -3.18588) );

pnt.push_back( gp_Pnt( 14.5, 0, 5.36814) );
pnt2.push_back( gp_Pnt( 14.5, 0, -5.36814) );
pnt3.push_back( gp_Pnt( 89.0848, 300, 3.22088) );
pnt4.push_back( gp_Pnt( 89.0848, 300, -3.22088) );

pnt.push_back( gp_Pnt( 15, 0, 5.42345) );
pnt2.push_back( gp_Pnt( 15, 0, -5.42345) );
pnt3.push_back( gp_Pnt( 89.3848, 300, 3.25407) );
pnt4.push_back( gp_Pnt( 89.3848, 300, -3.25407) );

pnt.push_back( gp_Pnt( 15.5, 0, 5.47586) );
pnt2.push_back( gp_Pnt( 15.5, 0, -5.47586) );
pnt3.push_back( gp_Pnt( 89.6848, 300, 3.28552) );
pnt4.push_back( gp_Pnt( 89.6848, 300, -3.28552) );

pnt.push_back( gp_Pnt( 16, 0, 5.52547) );
pnt2.push_back( gp_Pnt( 16, 0, -5.52547) );
pnt3.push_back( gp_Pnt( 89.9848, 300, 3.31528) );
pnt4.push_back( gp_Pnt( 89.9848, 300, -3.31528) );

pnt.push_back( gp_Pnt( 16.5, 0, 5.57238) );
pnt2.push_back( gp_Pnt( 16.5, 0, -5.57238) );
pnt3.push_back( gp_Pnt( 90.2848, 300, 3.34343) );
pnt4.push_back( gp_Pnt( 90.2848, 300, -3.34343) );

pnt.push_back( gp_Pnt( 17, 0, 5.61667) );
pnt2.push_back( gp_Pnt( 17, 0, -5.61667) );
pnt3.push_back( gp_Pnt( 90.5848, 300, 3.37) );
pnt4.push_back( gp_Pnt( 90.5848, 300, -3.37) );

pnt.push_back( gp_Pnt( 17.5, 0, 5.65843) );
pnt2.push_back( gp_Pnt( 17.5, 0, -5.65843) );
pnt3.push_back( gp_Pnt( 90.8848, 300, 3.39506) );
pnt4.push_back( gp_Pnt( 90.8848, 300, -3.39506) );

pnt.push_back( gp_Pnt( 18, 0, 5.69774) );
pnt2.push_back( gp_Pnt( 18, 0, -5.69774) );
pnt3.push_back( gp_Pnt( 91.1848, 300, 3.41864) );
pnt4.push_back( gp_Pnt( 91.1848, 300, -3.41864) );

pnt.push_back( gp_Pnt( 18.5, 0, 5.73468) );
pnt2.push_back( gp_Pnt( 18.5, 0, -5.73468) );
pnt3.push_back( gp_Pnt( 91.4848, 300, 3.44081) );
pnt4.push_back( gp_Pnt( 91.4848, 300, -3.44081) );

pnt.push_back( gp_Pnt( 19, 0, 5.76931) );
pnt2.push_back( gp_Pnt( 19, 0, -5.76931) );
pnt3.push_back( gp_Pnt( 91.7848, 300, 3.46159) );
pnt4.push_back( gp_Pnt( 91.7848, 300, -3.46159) );

pnt.push_back( gp_Pnt( 19.5, 0, 5.80172) );
pnt2.push_back( gp_Pnt( 19.5, 0, -5.80172) );
pnt3.push_back( gp_Pnt( 92.0848, 300, 3.48103) );
pnt4.push_back( gp_Pnt( 92.0848, 300, -3.48103) );

pnt.push_back( gp_Pnt( 20, 0, 5.83195) );
pnt2.push_back( gp_Pnt( 20, 0, -5.83195) );
pnt3.push_back( gp_Pnt( 92.3848, 300, 3.49917) );
pnt4.push_back( gp_Pnt( 92.3848, 300, -3.49917) );

pnt.push_back( gp_Pnt( 20.5, 0, 5.86008) );
pnt2.push_back( gp_Pnt( 20.5, 0, -5.86008) );
pnt3.push_back( gp_Pnt( 92.6848, 300, 3.51605) );
pnt4.push_back( gp_Pnt( 92.6848, 300, -3.51605) );

pnt.push_back( gp_Pnt( 21, 0, 5.88616) );
pnt2.push_back( gp_Pnt( 21, 0, -5.88616) );
pnt3.push_back( gp_Pnt( 92.9848, 300, 3.5317) );
pnt4.push_back( gp_Pnt( 92.9848, 300, -3.5317) );

pnt.push_back( gp_Pnt( 21.5, 0, 5.91026) );
pnt2.push_back( gp_Pnt( 21.5, 0, -5.91026) );
pnt3.push_back( gp_Pnt( 93.2848, 300, 3.54616) );
pnt4.push_back( gp_Pnt( 93.2848, 300, -3.54616) );

pnt.push_back( gp_Pnt( 22, 0, 5.93242) );
pnt2.push_back( gp_Pnt( 22, 0, -5.93242) );
pnt3.push_back( gp_Pnt( 93.5848, 300, 3.55945) );
pnt4.push_back( gp_Pnt( 93.5848, 300, -3.55945) );

pnt.push_back( gp_Pnt( 22.5, 0, 5.95269) );
pnt2.push_back( gp_Pnt( 22.5, 0, -5.95269) );
pnt3.push_back( gp_Pnt( 93.8848, 300, 3.57162) );
pnt4.push_back( gp_Pnt( 93.8848, 300, -3.57162) );

pnt.push_back( gp_Pnt( 23, 0, 5.97114) );
pnt2.push_back( gp_Pnt( 23, 0, -5.97114) );
pnt3.push_back( gp_Pnt( 94.1848, 300, 3.58268) );
pnt4.push_back( gp_Pnt( 94.1848, 300, -3.58268) );

pnt.push_back( gp_Pnt( 23.5, 0, 5.9878) );
pnt2.push_back( gp_Pnt( 23.5, 0, -5.9878) );
pnt3.push_back( gp_Pnt( 94.4848, 300, 3.59268) );
pnt4.push_back( gp_Pnt( 94.4848, 300, -3.59268) );

pnt.push_back( gp_Pnt( 24, 0, 6.00272) );
pnt2.push_back( gp_Pnt( 24, 0, -6.00272) );
pnt3.push_back( gp_Pnt( 94.7848, 300, 3.60163) );
pnt4.push_back( gp_Pnt( 94.7848, 300, -3.60163) );

pnt.push_back( gp_Pnt( 24.5, 0, 6.01595) );
pnt2.push_back( gp_Pnt( 24.5, 0, -6.01595) );
pnt3.push_back( gp_Pnt( 95.0848, 300, 3.60957) );
pnt4.push_back( gp_Pnt( 95.0848, 300, -3.60957) );

pnt.push_back( gp_Pnt( 25, 0, 6.02754) );
pnt2.push_back( gp_Pnt( 25, 0, -6.02754) );
pnt3.push_back( gp_Pnt( 95.3848, 300, 3.61652) );
pnt4.push_back( gp_Pnt( 95.3848, 300, -3.61652) );

pnt.push_back( gp_Pnt( 25.5, 0, 6.03752) );
pnt2.push_back( gp_Pnt( 25.5, 0, -6.03752) );
pnt3.push_back( gp_Pnt( 95.6848, 300, 3.62251) );
pnt4.push_back( gp_Pnt( 95.6848, 300, -3.62251) );

pnt.push_back( gp_Pnt( 26, 0, 6.04593) );
pnt2.push_back( gp_Pnt( 26, 0, -6.04593) );
pnt3.push_back( gp_Pnt( 95.9848, 300, 3.62756) );
pnt4.push_back( gp_Pnt( 95.9848, 300, -3.62756) );

pnt.push_back( gp_Pnt( 26.5, 0, 6.05283) );
pnt2.push_back( gp_Pnt( 26.5, 0, -6.05283) );
pnt3.push_back( gp_Pnt( 96.2848, 300, 3.6317) );
pnt4.push_back( gp_Pnt( 96.2848, 300, -3.6317) );

pnt.push_back( gp_Pnt( 27, 0, 6.05823) );
pnt2.push_back( gp_Pnt( 27, 0, -6.05823) );
pnt3.push_back( gp_Pnt( 96.5848, 300, 3.63494) );
pnt4.push_back( gp_Pnt( 96.5848, 300, -3.63494) );

pnt.push_back( gp_Pnt( 27.5, 0, 6.06219) );
pnt2.push_back( gp_Pnt( 27.5, 0, -6.06219) );
pnt3.push_back( gp_Pnt( 96.8848, 300, 3.63732) );
pnt4.push_back( gp_Pnt( 96.8848, 300, -3.63732) );

pnt.push_back( gp_Pnt( 28, 0, 6.06474) );
pnt2.push_back( gp_Pnt( 28, 0, -6.06474) );
pnt3.push_back( gp_Pnt( 97.1848, 300, 3.63885) );
pnt4.push_back( gp_Pnt( 97.1848, 300, -3.63885) );

pnt.push_back( gp_Pnt( 28.5, 0, 6.06592) );
pnt2.push_back( gp_Pnt( 28.5, 0, -6.06592) );
pnt3.push_back( gp_Pnt( 97.4848, 300, 3.63955) );
pnt4.push_back( gp_Pnt( 97.4848, 300, -3.63955) );

pnt.push_back( gp_Pnt( 29, 0, 6.06575) );
pnt2.push_back( gp_Pnt( 29, 0, -6.06575) );
pnt3.push_back( gp_Pnt( 97.7848, 300, 3.63945) );
pnt4.push_back( gp_Pnt( 97.7848, 300, -3.63945) );

pnt.push_back( gp_Pnt( 29.5, 0, 6.06428) );
pnt2.push_back( gp_Pnt( 29.5, 0, -6.06428) );
pnt3.push_back( gp_Pnt( 98.0848, 300, 3.63857) );
pnt4.push_back( gp_Pnt( 98.0848, 300, -3.63857) );

pnt.push_back( gp_Pnt( 30, 0, 6.06153) );
pnt2.push_back( gp_Pnt( 30, 0, -6.06153) );
pnt3.push_back( gp_Pnt( 98.3848, 300, 3.63692) );
pnt4.push_back( gp_Pnt( 98.3848, 300, -3.63692) );

pnt.push_back( gp_Pnt( 30.5, 0, 6.05754) );
pnt2.push_back( gp_Pnt( 30.5, 0, -6.05754) );
pnt3.push_back( gp_Pnt( 98.6848, 300, 3.63453) );
pnt4.push_back( gp_Pnt( 98.6848, 300, -3.63453) );

pnt.push_back( gp_Pnt( 31, 0, 6.05235) );
pnt2.push_back( gp_Pnt( 31, 0, -6.05235) );
pnt3.push_back( gp_Pnt( 98.9848, 300, 3.63141) );
pnt4.push_back( gp_Pnt( 98.9848, 300, -3.63141) );

pnt.push_back( gp_Pnt( 31.5, 0, 6.04597) );
pnt2.push_back( gp_Pnt( 31.5, 0, -6.04597) );
pnt3.push_back( gp_Pnt( 99.2848, 300, 3.62758) );
pnt4.push_back( gp_Pnt( 99.2848, 300, -3.62758) );

pnt.push_back( gp_Pnt( 32, 0, 6.03845) );
pnt2.push_back( gp_Pnt( 32, 0, -6.03845) );
pnt3.push_back( gp_Pnt( 99.5848, 300, 3.62307) );
pnt4.push_back( gp_Pnt( 99.5848, 300, -3.62307) );

pnt.push_back( gp_Pnt( 32.5, 0, 6.02981) );
pnt2.push_back( gp_Pnt( 32.5, 0, -6.02981) );
pnt3.push_back( gp_Pnt( 99.8848, 300, 3.61788) );
pnt4.push_back( gp_Pnt( 99.8848, 300, -3.61788) );

pnt.push_back( gp_Pnt( 33, 0, 6.02008) );
pnt2.push_back( gp_Pnt( 33, 0, -6.02008) );
pnt3.push_back( gp_Pnt( 100.185, 300, 3.61205) );
pnt4.push_back( gp_Pnt( 100.185, 300, -3.61205) );

pnt.push_back( gp_Pnt( 33.5, 0, 6.00928) );
pnt2.push_back( gp_Pnt( 33.5, 0, -6.00928) );
pnt3.push_back( gp_Pnt( 100.485, 300, 3.60557) );
pnt4.push_back( gp_Pnt( 100.485, 300, -3.60557) );

pnt.push_back( gp_Pnt( 34, 0, 5.99745) );
pnt2.push_back( gp_Pnt( 34, 0, -5.99745) );
pnt3.push_back( gp_Pnt( 100.785, 300, 3.59847) );
pnt4.push_back( gp_Pnt( 100.785, 300, -3.59847) );

pnt.push_back( gp_Pnt( 34.5, 0, 5.98462) );
pnt2.push_back( gp_Pnt( 34.5, 0, -5.98462) );
pnt3.push_back( gp_Pnt( 101.085, 300, 3.59077) );
pnt4.push_back( gp_Pnt( 101.085, 300, -3.59077) );

pnt.push_back( gp_Pnt( 35, 0, 5.9708) );
pnt2.push_back( gp_Pnt( 35, 0, -5.9708) );
pnt3.push_back( gp_Pnt( 101.385, 300, 3.58248) );
pnt4.push_back( gp_Pnt( 101.385, 300, -3.58248) );

pnt.push_back( gp_Pnt( 35.5, 0, 5.95603) );
pnt2.push_back( gp_Pnt( 35.5, 0, -5.95603) );
pnt3.push_back( gp_Pnt( 101.685, 300, 3.57362) );
pnt4.push_back( gp_Pnt( 101.685, 300, -3.57362) );

pnt.push_back( gp_Pnt( 36, 0, 5.94032) );
pnt2.push_back( gp_Pnt( 36, 0, -5.94032) );
pnt3.push_back( gp_Pnt( 101.985, 300, 3.56419) );
pnt4.push_back( gp_Pnt( 101.985, 300, -3.56419) );

pnt.push_back( gp_Pnt( 36.5, 0, 5.92372) );
pnt2.push_back( gp_Pnt( 36.5, 0, -5.92372) );
pnt3.push_back( gp_Pnt( 102.285, 300, 3.55423) );
pnt4.push_back( gp_Pnt( 102.285, 300, -3.55423) );

pnt.push_back( gp_Pnt( 37, 0, 5.90623) );
pnt2.push_back( gp_Pnt( 37, 0, -5.90623) );
pnt3.push_back( gp_Pnt( 102.585, 300, 3.54374) );
pnt4.push_back( gp_Pnt( 102.585, 300, -3.54374) );

pnt.push_back( gp_Pnt( 37.5, 0, 5.88788) );
pnt2.push_back( gp_Pnt( 37.5, 0, -5.88788) );
pnt3.push_back( gp_Pnt( 102.885, 300, 3.53273) );
pnt4.push_back( gp_Pnt( 102.885, 300, -3.53273) );

pnt.push_back( gp_Pnt( 38, 0, 5.8687) );
pnt2.push_back( gp_Pnt( 38, 0, -5.8687) );
pnt3.push_back( gp_Pnt( 103.185, 300, 3.52122) );
pnt4.push_back( gp_Pnt( 103.185, 300, -3.52122) );

pnt.push_back( gp_Pnt( 38.5, 0, 5.84871) );
pnt2.push_back( gp_Pnt( 38.5, 0, -5.84871) );
pnt3.push_back( gp_Pnt( 103.485, 300, 3.50922) );
pnt4.push_back( gp_Pnt( 103.485, 300, -3.50922) );

pnt.push_back( gp_Pnt( 39, 0, 5.82793) );
pnt2.push_back( gp_Pnt( 39, 0, -5.82793) );
pnt3.push_back( gp_Pnt( 103.785, 300, 3.49676) );
pnt4.push_back( gp_Pnt( 103.785, 300, -3.49676) );

pnt.push_back( gp_Pnt( 39.5, 0, 5.80638) );
pnt2.push_back( gp_Pnt( 39.5, 0, -5.80638) );
pnt3.push_back( gp_Pnt( 104.085, 300, 3.48383) );
pnt4.push_back( gp_Pnt( 104.085, 300, -3.48383) );

pnt.push_back( gp_Pnt( 40, 0, 5.78408) );
pnt2.push_back( gp_Pnt( 40, 0, -5.78408) );
pnt3.push_back( gp_Pnt( 104.385, 300, 3.47045) );
pnt4.push_back( gp_Pnt( 104.385, 300, -3.47045) );

pnt.push_back( gp_Pnt( 40.5, 0, 5.76106) );
pnt2.push_back( gp_Pnt( 40.5, 0, -5.76106) );
pnt3.push_back( gp_Pnt( 104.685, 300, 3.45664) );
pnt4.push_back( gp_Pnt( 104.685, 300, -3.45664) );

pnt.push_back( gp_Pnt( 41, 0, 5.73734) );
pnt2.push_back( gp_Pnt( 41, 0, -5.73734) );
pnt3.push_back( gp_Pnt( 104.985, 300, 3.4424) );
pnt4.push_back( gp_Pnt( 104.985, 300, -3.4424) );

pnt.push_back( gp_Pnt( 41.5, 0, 5.71293) );
pnt2.push_back( gp_Pnt( 41.5, 0, -5.71293) );
pnt3.push_back( gp_Pnt( 105.285, 300, 3.42776) );
pnt4.push_back( gp_Pnt( 105.285, 300, -3.42776) );

pnt.push_back( gp_Pnt( 42, 0, 5.68786) );
pnt2.push_back( gp_Pnt( 42, 0, -5.68786) );
pnt3.push_back( gp_Pnt( 105.585, 300, 3.41271) );
pnt4.push_back( gp_Pnt( 105.585, 300, -3.41271) );

pnt.push_back( gp_Pnt( 42.5, 0, 5.66214) );
pnt2.push_back( gp_Pnt( 42.5, 0, -5.66214) );
pnt3.push_back( gp_Pnt( 105.885, 300, 3.39728) );
pnt4.push_back( gp_Pnt( 105.885, 300, -3.39728) );

pnt.push_back( gp_Pnt( 43, 0, 5.63579) );
pnt2.push_back( gp_Pnt( 43, 0, -5.63579) );
pnt3.push_back( gp_Pnt( 106.185, 300, 3.38147) );
pnt4.push_back( gp_Pnt( 106.185, 300, -3.38147) );

pnt.push_back( gp_Pnt( 43.5, 0, 5.60883) );
pnt2.push_back( gp_Pnt( 43.5, 0, -5.60883) );
pnt3.push_back( gp_Pnt( 106.485, 300, 3.3653) );
pnt4.push_back( gp_Pnt( 106.485, 300, -3.3653) );

pnt.push_back( gp_Pnt( 44, 0, 5.58129) );
pnt2.push_back( gp_Pnt( 44, 0, -5.58129) );
pnt3.push_back( gp_Pnt( 106.785, 300, 3.34877) );
pnt4.push_back( gp_Pnt( 106.785, 300, -3.34877) );

pnt.push_back( gp_Pnt( 44.5, 0, 5.55317) );
pnt2.push_back( gp_Pnt( 44.5, 0, -5.55317) );
pnt3.push_back( gp_Pnt( 107.085, 300, 3.3319) );
pnt4.push_back( gp_Pnt( 107.085, 300, -3.3319) );

pnt.push_back( gp_Pnt( 45, 0, 5.52449) );
pnt2.push_back( gp_Pnt( 45, 0, -5.52449) );
pnt3.push_back( gp_Pnt( 107.385, 300, 3.31469) );
pnt4.push_back( gp_Pnt( 107.385, 300, -3.31469) );

pnt.push_back( gp_Pnt( 45.5, 0, 5.49527) );
pnt2.push_back( gp_Pnt( 45.5, 0, -5.49527) );
pnt3.push_back( gp_Pnt( 107.685, 300, 3.29716) );
pnt4.push_back( gp_Pnt( 107.685, 300, -3.29716) );

pnt.push_back( gp_Pnt( 46, 0, 5.46552) );
pnt2.push_back( gp_Pnt( 46, 0, -5.46552) );
pnt3.push_back( gp_Pnt( 107.985, 300, 3.27931) );
pnt4.push_back( gp_Pnt( 107.985, 300, -3.27931) );

pnt.push_back( gp_Pnt( 46.5, 0, 5.43527) );
pnt2.push_back( gp_Pnt( 46.5, 0, -5.43527) );
pnt3.push_back( gp_Pnt( 108.285, 300, 3.26116) );
pnt4.push_back( gp_Pnt( 108.285, 300, -3.26116) );

pnt.push_back( gp_Pnt( 47, 0, 5.40453) );
pnt2.push_back( gp_Pnt( 47, 0, -5.40453) );
pnt3.push_back( gp_Pnt( 108.585, 300, 3.24272) );
pnt4.push_back( gp_Pnt( 108.585, 300, -3.24272) );

pnt.push_back( gp_Pnt( 47.5, 0, 5.3733) );
pnt2.push_back( gp_Pnt( 47.5, 0, -5.3733) );
pnt3.push_back( gp_Pnt( 108.885, 300, 3.22398) );
pnt4.push_back( gp_Pnt( 108.885, 300, -3.22398) );

pnt.push_back( gp_Pnt( 48, 0, 5.34161) );
pnt2.push_back( gp_Pnt( 48, 0, -5.34161) );
pnt3.push_back( gp_Pnt( 109.185, 300, 3.20497) );
pnt4.push_back( gp_Pnt( 109.185, 300, -3.20497) );

pnt.push_back( gp_Pnt( 48.5, 0, 5.30947) );
pnt2.push_back( gp_Pnt( 48.5, 0, -5.30947) );
pnt3.push_back( gp_Pnt( 109.485, 300, 3.18568) );
pnt4.push_back( gp_Pnt( 109.485, 300, -3.18568) );

pnt.push_back( gp_Pnt( 49, 0, 5.2769) );
pnt2.push_back( gp_Pnt( 49, 0, -5.2769) );
pnt3.push_back( gp_Pnt( 109.785, 300, 3.16614) );
pnt4.push_back( gp_Pnt( 109.785, 300, -3.16614) );

pnt.push_back( gp_Pnt( 49.5, 0, 5.2439) );
pnt2.push_back( gp_Pnt( 49.5, 0, -5.2439) );
pnt3.push_back( gp_Pnt( 110.085, 300, 3.14634) );
pnt4.push_back( gp_Pnt( 110.085, 300, -3.14634) );

pnt.push_back( gp_Pnt( 50, 0, 5.21049) );
pnt2.push_back( gp_Pnt( 50, 0, -5.21049) );
pnt3.push_back( gp_Pnt( 110.385, 300, 3.1263) );
pnt4.push_back( gp_Pnt( 110.385, 300, -3.1263) );

pnt.push_back( gp_Pnt( 50.5, 0, 5.17669) );
pnt2.push_back( gp_Pnt( 50.5, 0, -5.17669) );
pnt3.push_back( gp_Pnt( 110.685, 300, 3.10601) );
pnt4.push_back( gp_Pnt( 110.685, 300, -3.10601) );

pnt.push_back( gp_Pnt( 51, 0, 5.14249) );
pnt2.push_back( gp_Pnt( 51, 0, -5.14249) );
pnt3.push_back( gp_Pnt( 110.985, 300, 3.0855) );
pnt4.push_back( gp_Pnt( 110.985, 300, -3.0855) );

pnt.push_back( gp_Pnt( 51.5, 0, 5.10793) );
pnt2.push_back( gp_Pnt( 51.5, 0, -5.10793) );
pnt3.push_back( gp_Pnt( 111.285, 300, 3.06476) );
pnt4.push_back( gp_Pnt( 111.285, 300, -3.06476) );

pnt.push_back( gp_Pnt( 52, 0, 5.07299) );
pnt2.push_back( gp_Pnt( 52, 0, -5.07299) );
pnt3.push_back( gp_Pnt( 111.585, 300, 3.0438) );
pnt4.push_back( gp_Pnt( 111.585, 300, -3.0438) );

pnt.push_back( gp_Pnt( 52.5, 0, 5.03771) );
pnt2.push_back( gp_Pnt( 52.5, 0, -5.03771) );
pnt3.push_back( gp_Pnt( 111.885, 300, 3.02263) );
pnt4.push_back( gp_Pnt( 111.885, 300, -3.02263) );

pnt.push_back( gp_Pnt( 53, 0, 5.00208) );
pnt2.push_back( gp_Pnt( 53, 0, -5.00208) );
pnt3.push_back( gp_Pnt( 112.185, 300, 3.00125) );
pnt4.push_back( gp_Pnt( 112.185, 300, -3.00125) );

pnt.push_back( gp_Pnt( 53.5, 0, 4.96613) );
pnt2.push_back( gp_Pnt( 53.5, 0, -4.96613) );
pnt3.push_back( gp_Pnt( 112.485, 300, 2.97968) );
pnt4.push_back( gp_Pnt( 112.485, 300, -2.97968) );

pnt.push_back( gp_Pnt( 54, 0, 4.92984) );
pnt2.push_back( gp_Pnt( 54, 0, -4.92984) );
pnt3.push_back( gp_Pnt( 112.785, 300, 2.95791) );
pnt4.push_back( gp_Pnt( 112.785, 300, -2.95791) );

pnt.push_back( gp_Pnt( 54.5, 0, 4.89325) );
pnt2.push_back( gp_Pnt( 54.5, 0, -4.89325) );
pnt3.push_back( gp_Pnt( 113.085, 300, 2.93595) );
pnt4.push_back( gp_Pnt( 113.085, 300, -2.93595) );

pnt.push_back( gp_Pnt( 55, 0, 4.85634) );
pnt2.push_back( gp_Pnt( 55, 0, -4.85634) );
pnt3.push_back( gp_Pnt( 113.385, 300, 2.91381) );
pnt4.push_back( gp_Pnt( 113.385, 300, -2.91381) );

pnt.push_back( gp_Pnt( 55.5, 0, 4.81914) );
pnt2.push_back( gp_Pnt( 55.5, 0, -4.81914) );
pnt3.push_back( gp_Pnt( 113.685, 300, 2.89149) );
pnt4.push_back( gp_Pnt( 113.685, 300, -2.89149) );

pnt.push_back( gp_Pnt( 56, 0, 4.78165) );
pnt2.push_back( gp_Pnt( 56, 0, -4.78165) );
pnt3.push_back( gp_Pnt( 113.985, 300, 2.86899) );
pnt4.push_back( gp_Pnt( 113.985, 300, -2.86899) );

pnt.push_back( gp_Pnt( 56.5, 0, 4.74388) );
pnt2.push_back( gp_Pnt( 56.5, 0, -4.74388) );
pnt3.push_back( gp_Pnt( 114.285, 300, 2.84633) );
pnt4.push_back( gp_Pnt( 114.285, 300, -2.84633) );

pnt.push_back( gp_Pnt( 57, 0, 4.70583) );
pnt2.push_back( gp_Pnt( 57, 0, -4.70583) );
pnt3.push_back( gp_Pnt( 114.585, 300, 2.8235) );
pnt4.push_back( gp_Pnt( 114.585, 300, -2.8235) );

pnt.push_back( gp_Pnt( 57.5, 0, 4.66752) );
pnt2.push_back( gp_Pnt( 57.5, 0, -4.66752) );
pnt3.push_back( gp_Pnt( 114.885, 300, 2.80051) );
pnt4.push_back( gp_Pnt( 114.885, 300, -2.80051) );

pnt.push_back( gp_Pnt( 58, 0, 4.62894) );
pnt2.push_back( gp_Pnt( 58, 0, -4.62894) );
pnt3.push_back( gp_Pnt( 115.185, 300, 2.77736) );
pnt4.push_back( gp_Pnt( 115.185, 300, -2.77736) );

pnt.push_back( gp_Pnt( 58.5, 0, 4.5901) );
pnt2.push_back( gp_Pnt( 58.5, 0, -4.5901) );
pnt3.push_back( gp_Pnt( 115.485, 300, 2.75406) );
pnt4.push_back( gp_Pnt( 115.485, 300, -2.75406) );

pnt.push_back( gp_Pnt( 59, 0, 4.55101) );
pnt2.push_back( gp_Pnt( 59, 0, -4.55101) );
pnt3.push_back( gp_Pnt( 115.785, 300, 2.7306) );
pnt4.push_back( gp_Pnt( 115.785, 300, -2.7306) );

pnt.push_back( gp_Pnt( 59.5, 0, 4.51167) );
pnt2.push_back( gp_Pnt( 59.5, 0, -4.51167) );
pnt3.push_back( gp_Pnt( 116.085, 300, 2.707) );
pnt4.push_back( gp_Pnt( 116.085, 300, -2.707) );

pnt.push_back( gp_Pnt( 60, 0, 4.47209) );
pnt2.push_back( gp_Pnt( 60, 0, -4.47209) );
pnt3.push_back( gp_Pnt( 116.385, 300, 2.68325) );
pnt4.push_back( gp_Pnt( 116.385, 300, -2.68325) );

pnt.push_back( gp_Pnt( 60.5, 0, 4.43227) );
pnt2.push_back( gp_Pnt( 60.5, 0, -4.43227) );
pnt3.push_back( gp_Pnt( 116.685, 300, 2.65936) );
pnt4.push_back( gp_Pnt( 116.685, 300, -2.65936) );

pnt.push_back( gp_Pnt( 61, 0, 4.39221) );
pnt2.push_back( gp_Pnt( 61, 0, -4.39221) );
pnt3.push_back( gp_Pnt( 116.985, 300, 2.63532) );
pnt4.push_back( gp_Pnt( 116.985, 300, -2.63532) );

pnt.push_back( gp_Pnt( 61.5, 0, 4.35191) );
pnt2.push_back( gp_Pnt( 61.5, 0, -4.35191) );
pnt3.push_back( gp_Pnt( 117.285, 300, 2.61115) );
pnt4.push_back( gp_Pnt( 117.285, 300, -2.61115) );

pnt.push_back( gp_Pnt( 62, 0, 4.31139) );
pnt2.push_back( gp_Pnt( 62, 0, -4.31139) );
pnt3.push_back( gp_Pnt( 117.585, 300, 2.58683) );
pnt4.push_back( gp_Pnt( 117.585, 300, -2.58683) );

pnt.push_back( gp_Pnt( 62.5, 0, 4.27064) );
pnt2.push_back( gp_Pnt( 62.5, 0, -4.27064) );
pnt3.push_back( gp_Pnt( 117.885, 300, 2.56238) );
pnt4.push_back( gp_Pnt( 117.885, 300, -2.56238) );

pnt.push_back( gp_Pnt( 63, 0, 4.22966) );
pnt2.push_back( gp_Pnt( 63, 0, -4.22966) );
pnt3.push_back( gp_Pnt( 118.185, 300, 2.53779) );
pnt4.push_back( gp_Pnt( 118.185, 300, -2.53779) );

pnt.push_back( gp_Pnt( 63.5, 0, 4.18845) );
pnt2.push_back( gp_Pnt( 63.5, 0, -4.18845) );
pnt3.push_back( gp_Pnt( 118.485, 300, 2.51307) );
pnt4.push_back( gp_Pnt( 118.485, 300, -2.51307) );

pnt.push_back( gp_Pnt( 64, 0, 4.14702) );
pnt2.push_back( gp_Pnt( 64, 0, -4.14702) );
pnt3.push_back( gp_Pnt( 118.785, 300, 2.48821) );
pnt4.push_back( gp_Pnt( 118.785, 300, -2.48821) );

pnt.push_back( gp_Pnt( 64.5, 0, 4.10536) );
pnt2.push_back( gp_Pnt( 64.5, 0, -4.10536) );
pnt3.push_back( gp_Pnt( 119.085, 300, 2.46322) );
pnt4.push_back( gp_Pnt( 119.085, 300, -2.46322) );

pnt.push_back( gp_Pnt( 65, 0, 4.06347) );
pnt2.push_back( gp_Pnt( 65, 0, -4.06347) );
pnt3.push_back( gp_Pnt( 119.385, 300, 2.43808) );
pnt4.push_back( gp_Pnt( 119.385, 300, -2.43808) );

pnt.push_back( gp_Pnt( 65.5, 0, 4.02136) );
pnt2.push_back( gp_Pnt( 65.5, 0, -4.02136) );
pnt3.push_back( gp_Pnt( 119.685, 300, 2.41282) );
pnt4.push_back( gp_Pnt( 119.685, 300, -2.41282) );

pnt.push_back( gp_Pnt( 66, 0, 3.97902) );
pnt2.push_back( gp_Pnt( 66, 0, -3.97902) );
pnt3.push_back( gp_Pnt( 119.985, 300, 2.38741) );
pnt4.push_back( gp_Pnt( 119.985, 300, -2.38741) );

pnt.push_back( gp_Pnt( 66.5, 0, 3.93645) );
pnt2.push_back( gp_Pnt( 66.5, 0, -3.93645) );
pnt3.push_back( gp_Pnt( 120.285, 300, 2.36187) );
pnt4.push_back( gp_Pnt( 120.285, 300, -2.36187) );

pnt.push_back( gp_Pnt( 67, 0, 3.89365) );
pnt2.push_back( gp_Pnt( 67, 0, -3.89365) );
pnt3.push_back( gp_Pnt( 120.585, 300, 2.33619) );
pnt4.push_back( gp_Pnt( 120.585, 300, -2.33619) );

pnt.push_back( gp_Pnt( 67.5, 0, 3.85061) );
pnt2.push_back( gp_Pnt( 67.5, 0, -3.85061) );
pnt3.push_back( gp_Pnt( 120.885, 300, 2.31037) );
pnt4.push_back( gp_Pnt( 120.885, 300, -2.31037) );

pnt.push_back( gp_Pnt( 68, 0, 3.80733) );
pnt2.push_back( gp_Pnt( 68, 0, -3.80733) );
pnt3.push_back( gp_Pnt( 121.185, 300, 2.2844) );
pnt4.push_back( gp_Pnt( 121.185, 300, -2.2844) );

pnt.push_back( gp_Pnt( 68.5, 0, 3.76381) );
pnt2.push_back( gp_Pnt( 68.5, 0, -3.76381) );
pnt3.push_back( gp_Pnt( 121.485, 300, 2.25829) );
pnt4.push_back( gp_Pnt( 121.485, 300, -2.25829) );

pnt.push_back( gp_Pnt( 69, 0, 3.72005) );
pnt2.push_back( gp_Pnt( 69, 0, -3.72005) );
pnt3.push_back( gp_Pnt( 121.785, 300, 2.23203) );
pnt4.push_back( gp_Pnt( 121.785, 300, -2.23203) );

pnt.push_back( gp_Pnt( 69.5, 0, 3.67604) );
pnt2.push_back( gp_Pnt( 69.5, 0, -3.67604) );
pnt3.push_back( gp_Pnt( 122.085, 300, 2.20562) );
pnt4.push_back( gp_Pnt( 122.085, 300, -2.20562) );

pnt.push_back( gp_Pnt( 70, 0, 3.63177) );
pnt2.push_back( gp_Pnt( 70, 0, -3.63177) );
pnt3.push_back( gp_Pnt( 122.385, 300, 2.17906) );
pnt4.push_back( gp_Pnt( 122.385, 300, -2.17906) );

pnt.push_back( gp_Pnt( 70.5, 0, 3.58723) );
pnt2.push_back( gp_Pnt( 70.5, 0, -3.58723) );
pnt3.push_back( gp_Pnt( 122.685, 300, 2.15234) );
pnt4.push_back( gp_Pnt( 122.685, 300, -2.15234) );

pnt.push_back( gp_Pnt( 71, 0, 3.54243) );
pnt2.push_back( gp_Pnt( 71, 0, -3.54243) );
pnt3.push_back( gp_Pnt( 122.985, 300, 2.12546) );
pnt4.push_back( gp_Pnt( 122.985, 300, -2.12546) );

pnt.push_back( gp_Pnt( 71.5, 0, 3.49736) );
pnt2.push_back( gp_Pnt( 71.5, 0, -3.49736) );
pnt3.push_back( gp_Pnt( 123.285, 300, 2.09842) );
pnt4.push_back( gp_Pnt( 123.285, 300, -2.09842) );

pnt.push_back( gp_Pnt( 72, 0, 3.45201) );
pnt2.push_back( gp_Pnt( 72, 0, -3.45201) );
pnt3.push_back( gp_Pnt( 123.585, 300, 2.0712) );
pnt4.push_back( gp_Pnt( 123.585, 300, -2.0712) );

pnt.push_back( gp_Pnt( 72.5, 0, 3.40636) );
pnt2.push_back( gp_Pnt( 72.5, 0, -3.40636) );
pnt3.push_back( gp_Pnt( 123.885, 300, 2.04382) );
pnt4.push_back( gp_Pnt( 123.885, 300, -2.04382) );

pnt.push_back( gp_Pnt( 73, 0, 3.36042) );
pnt2.push_back( gp_Pnt( 73, 0, -3.36042) );
pnt3.push_back( gp_Pnt( 124.185, 300, 2.01625) );
pnt4.push_back( gp_Pnt( 124.185, 300, -2.01625) );

pnt.push_back( gp_Pnt( 73.5, 0, 3.31418) );
pnt2.push_back( gp_Pnt( 73.5, 0, -3.31418) );
pnt3.push_back( gp_Pnt( 124.485, 300, 1.98851) );
pnt4.push_back( gp_Pnt( 124.485, 300, -1.98851) );

pnt.push_back( gp_Pnt( 74, 0, 3.26762) );
pnt2.push_back( gp_Pnt( 74, 0, -3.26762) );
pnt3.push_back( gp_Pnt( 124.785, 300, 1.96057) );
pnt4.push_back( gp_Pnt( 124.785, 300, -1.96057) );

pnt.push_back( gp_Pnt( 74.5, 0, 3.22073) );
pnt2.push_back( gp_Pnt( 74.5, 0, -3.22073) );
pnt3.push_back( gp_Pnt( 125.085, 300, 1.93244) );
pnt4.push_back( gp_Pnt( 125.085, 300, -1.93244) );

pnt.push_back( gp_Pnt( 75, 0, 3.17351) );
pnt2.push_back( gp_Pnt( 75, 0, -3.17351) );
pnt3.push_back( gp_Pnt( 125.385, 300, 1.90411) );
pnt4.push_back( gp_Pnt( 125.385, 300, -1.90411) );

pnt.push_back( gp_Pnt( 75.5, 0, 3.12594) );
pnt2.push_back( gp_Pnt( 75.5, 0, -3.12594) );
pnt3.push_back( gp_Pnt( 125.685, 300, 1.87557) );
pnt4.push_back( gp_Pnt( 125.685, 300, -1.87557) );

pnt.push_back( gp_Pnt( 76, 0, 3.07802) );
pnt2.push_back( gp_Pnt( 76, 0, -3.07802) );
pnt3.push_back( gp_Pnt( 125.985, 300, 1.84681) );
pnt4.push_back( gp_Pnt( 125.985, 300, -1.84681) );

pnt.push_back( gp_Pnt( 76.5, 0, 3.02973) );
pnt2.push_back( gp_Pnt( 76.5, 0, -3.02973) );
pnt3.push_back( gp_Pnt( 126.285, 300, 1.81784) );
pnt4.push_back( gp_Pnt( 126.285, 300, -1.81784) );

pnt.push_back( gp_Pnt( 77, 0, 2.98106) );
pnt2.push_back( gp_Pnt( 77, 0, -2.98106) );
pnt3.push_back( gp_Pnt( 126.585, 300, 1.78864) );
pnt4.push_back( gp_Pnt( 126.585, 300, -1.78864) );

pnt.push_back( gp_Pnt( 77.5, 0, 2.93199) );
pnt2.push_back( gp_Pnt( 77.5, 0, -2.93199) );
pnt3.push_back( gp_Pnt( 126.885, 300, 1.7592) );
pnt4.push_back( gp_Pnt( 126.885, 300, -1.7592) );

pnt.push_back( gp_Pnt( 78, 0, 2.88252) );
pnt2.push_back( gp_Pnt( 78, 0, -2.88252) );
pnt3.push_back( gp_Pnt( 127.185, 300, 1.72951) );
pnt4.push_back( gp_Pnt( 127.185, 300, -1.72951) );

pnt.push_back( gp_Pnt( 78.5, 0, 2.83263) );
pnt2.push_back( gp_Pnt( 78.5, 0, -2.83263) );
pnt3.push_back( gp_Pnt( 127.485, 300, 1.69958) );
pnt4.push_back( gp_Pnt( 127.485, 300, -1.69958) );

pnt.push_back( gp_Pnt( 79, 0, 2.7823) );
pnt2.push_back( gp_Pnt( 79, 0, -2.7823) );
pnt3.push_back( gp_Pnt( 127.785, 300, 1.66938) );
pnt4.push_back( gp_Pnt( 127.785, 300, -1.66938) );

pnt.push_back( gp_Pnt( 79.5, 0, 2.73152) );
pnt2.push_back( gp_Pnt( 79.5, 0, -2.73152) );
pnt3.push_back( gp_Pnt( 128.085, 300, 1.63891) );
pnt4.push_back( gp_Pnt( 128.085, 300, -1.63891) );

pnt.push_back( gp_Pnt( 80, 0, 2.68028) );
pnt2.push_back( gp_Pnt( 80, 0, -2.68028) );
pnt3.push_back( gp_Pnt( 128.385, 300, 1.60817) );
pnt4.push_back( gp_Pnt( 128.385, 300, -1.60817) );

pnt.push_back( gp_Pnt( 80.5, 0, 2.62855) );
pnt2.push_back( gp_Pnt( 80.5, 0, -2.62855) );
pnt3.push_back( gp_Pnt( 128.685, 300, 1.57713) );
pnt4.push_back( gp_Pnt( 128.685, 300, -1.57713) );

pnt.push_back( gp_Pnt( 81, 0, 2.57632) );
pnt2.push_back( gp_Pnt( 81, 0, -2.57632) );
pnt3.push_back( gp_Pnt( 128.985, 300, 1.54579) );
pnt4.push_back( gp_Pnt( 128.985, 300, -1.54579) );

pnt.push_back( gp_Pnt( 81.5, 0, 2.52358) );
pnt2.push_back( gp_Pnt( 81.5, 0, -2.52358) );
pnt3.push_back( gp_Pnt( 129.285, 300, 1.51415) );
pnt4.push_back( gp_Pnt( 129.285, 300, -1.51415) );

pnt.push_back( gp_Pnt( 82, 0, 2.47031) );
pnt2.push_back( gp_Pnt( 82, 0, -2.47031) );
pnt3.push_back( gp_Pnt( 129.585, 300, 1.48219) );
pnt4.push_back( gp_Pnt( 129.585, 300, -1.48219) );

pnt.push_back( gp_Pnt( 82.5, 0, 2.41648) );
pnt2.push_back( gp_Pnt( 82.5, 0, -2.41648) );
pnt3.push_back( gp_Pnt( 129.885, 300, 1.44989) );
pnt4.push_back( gp_Pnt( 129.885, 300, -1.44989) );

pnt.push_back( gp_Pnt( 83, 0, 2.36209) );
pnt2.push_back( gp_Pnt( 83, 0, -2.36209) );
pnt3.push_back( gp_Pnt( 130.185, 300, 1.41725) );
pnt4.push_back( gp_Pnt( 130.185, 300, -1.41725) );

pnt.push_back( gp_Pnt( 83.5, 0, 2.3071) );
pnt2.push_back( gp_Pnt( 83.5, 0, -2.3071) );
pnt3.push_back( gp_Pnt( 130.485, 300, 1.38426) );
pnt4.push_back( gp_Pnt( 130.485, 300, -1.38426) );

pnt.push_back( gp_Pnt( 84, 0, 2.2515) );
pnt2.push_back( gp_Pnt( 84, 0, -2.2515) );
pnt3.push_back( gp_Pnt( 130.785, 300, 1.3509) );
pnt4.push_back( gp_Pnt( 130.785, 300, -1.3509) );

pnt.push_back( gp_Pnt( 84.5, 0, 2.19528) );
pnt2.push_back( gp_Pnt( 84.5, 0, -2.19528) );
pnt3.push_back( gp_Pnt( 131.085, 300, 1.31717) );
pnt4.push_back( gp_Pnt( 131.085, 300, -1.31717) );

pnt.push_back( gp_Pnt( 85, 0, 2.1384) );
pnt2.push_back( gp_Pnt( 85, 0, -2.1384) );
pnt3.push_back( gp_Pnt( 131.385, 300, 1.28304) );
pnt4.push_back( gp_Pnt( 131.385, 300, -1.28304) );

pnt.push_back( gp_Pnt( 85.5, 0, 2.08085) );
pnt2.push_back( gp_Pnt( 85.5, 0, -2.08085) );
pnt3.push_back( gp_Pnt( 131.685, 300, 1.24851) );
pnt4.push_back( gp_Pnt( 131.685, 300, -1.24851) );

pnt.push_back( gp_Pnt( 86, 0, 2.0226) );
pnt2.push_back( gp_Pnt( 86, 0, -2.0226) );
pnt3.push_back( gp_Pnt( 131.985, 300, 1.21356) );
pnt4.push_back( gp_Pnt( 131.985, 300, -1.21356) );

pnt.push_back( gp_Pnt( 86.5, 0, 1.96363) );
pnt2.push_back( gp_Pnt( 86.5, 0, -1.96363) );
pnt3.push_back( gp_Pnt( 132.285, 300, 1.17818) );
pnt4.push_back( gp_Pnt( 132.285, 300, -1.17818) );

pnt.push_back( gp_Pnt( 87, 0, 1.90392) );
pnt2.push_back( gp_Pnt( 87, 0, -1.90392) );
pnt3.push_back( gp_Pnt( 132.585, 300, 1.14235) );
pnt4.push_back( gp_Pnt( 132.585, 300, -1.14235) );

pnt.push_back( gp_Pnt( 87.5, 0, 1.84345) );
pnt2.push_back( gp_Pnt( 87.5, 0, -1.84345) );
pnt3.push_back( gp_Pnt( 132.885, 300, 1.10607) );
pnt4.push_back( gp_Pnt( 132.885, 300, -1.10607) );

pnt.push_back( gp_Pnt( 88, 0, 1.78218) );
pnt2.push_back( gp_Pnt( 88, 0, -1.78218) );
pnt3.push_back( gp_Pnt( 133.185, 300, 1.06931) );
pnt4.push_back( gp_Pnt( 133.185, 300, -1.06931) );

pnt.push_back( gp_Pnt( 88.5, 0, 1.7201) );
pnt2.push_back( gp_Pnt( 88.5, 0, -1.7201) );
pnt3.push_back( gp_Pnt( 133.485, 300, 1.03206) );
pnt4.push_back( gp_Pnt( 133.485, 300, -1.03206) );

pnt.push_back( gp_Pnt( 89, 0, 1.65718) );
pnt2.push_back( gp_Pnt( 89, 0, -1.65718) );
pnt3.push_back( gp_Pnt( 133.785, 300, 0.994306) );
pnt4.push_back( gp_Pnt( 133.785, 300, -0.994306) );

pnt.push_back( gp_Pnt( 89.5, 0, 1.59339) );
pnt2.push_back( gp_Pnt( 89.5, 0, -1.59339) );
pnt3.push_back( gp_Pnt( 134.085, 300, 0.956031) );
pnt4.push_back( gp_Pnt( 134.085, 300, -0.956031) );

pnt.push_back( gp_Pnt( 90, 0, 1.5287) );
pnt2.push_back( gp_Pnt( 90, 0, -1.5287) );
pnt3.push_back( gp_Pnt( 134.385, 300, 0.917219) );
pnt4.push_back( gp_Pnt( 134.385, 300, -0.917219) );

pnt.push_back( gp_Pnt( 90.5, 0, 1.46309) );
pnt2.push_back( gp_Pnt( 90.5, 0, -1.46309) );
pnt3.push_back( gp_Pnt( 134.685, 300, 0.877854) );
pnt4.push_back( gp_Pnt( 134.685, 300, -0.877854) );

pnt.push_back( gp_Pnt( 91, 0, 1.39653) );
pnt2.push_back( gp_Pnt( 91, 0, -1.39653) );
pnt3.push_back( gp_Pnt( 134.985, 300, 0.837918) );
pnt4.push_back( gp_Pnt( 134.985, 300, -0.837918) );

pnt.push_back( gp_Pnt( 91.5, 0, 1.32899) );
pnt2.push_back( gp_Pnt( 91.5, 0, -1.32899) );
pnt3.push_back( gp_Pnt( 135.285, 300, 0.797394) );
pnt4.push_back( gp_Pnt( 135.285, 300, -0.797394) );

pnt.push_back( gp_Pnt( 92, 0, 1.26044) );
pnt2.push_back( gp_Pnt( 92, 0, -1.26044) );
pnt3.push_back( gp_Pnt( 135.585, 300, 0.756264) );
pnt4.push_back( gp_Pnt( 135.585, 300, -0.756264) );

pnt.push_back( gp_Pnt( 92.5, 0, 1.19085) );
pnt2.push_back( gp_Pnt( 92.5, 0, -1.19085) );
pnt3.push_back( gp_Pnt( 135.885, 300, 0.714511) );
pnt4.push_back( gp_Pnt( 135.885, 300, -0.714511) );

pnt.push_back( gp_Pnt( 93, 0, 1.12019) );
pnt2.push_back( gp_Pnt( 93, 0, -1.12019) );
pnt3.push_back( gp_Pnt( 136.185, 300, 0.672117) );
pnt4.push_back( gp_Pnt( 136.185, 300, -0.672117) );

pnt.push_back( gp_Pnt( 93.5, 0, 1.04844) );
pnt2.push_back( gp_Pnt( 93.5, 0, -1.04844) );
pnt3.push_back( gp_Pnt( 136.485, 300, 0.629061) );
pnt4.push_back( gp_Pnt( 136.485, 300, -0.629061) );

pnt.push_back( gp_Pnt( 94, 0, 0.975543) );
pnt2.push_back( gp_Pnt( 94, 0, -0.975543) );
pnt3.push_back( gp_Pnt( 136.785, 300, 0.585326) );
pnt4.push_back( gp_Pnt( 136.785, 300, -0.585326) );

pnt.push_back( gp_Pnt( 94.5, 0, 0.901485) );
pnt2.push_back( gp_Pnt( 94.5, 0, -0.901485) );
pnt3.push_back( gp_Pnt( 137.085, 300, 0.540891) );
pnt4.push_back( gp_Pnt( 137.085, 300, -0.540891) );

pnt.push_back( gp_Pnt( 95, 0, 0.826229) );
pnt2.push_back( gp_Pnt( 95, 0, -0.826229) );
pnt3.push_back( gp_Pnt( 137.385, 300, 0.495737) );
pnt4.push_back( gp_Pnt( 137.385, 300, -0.495737) );

pnt.push_back( gp_Pnt( 95.5, 0, 0.74974) );
pnt2.push_back( gp_Pnt( 95.5, 0, -0.74974) );
pnt3.push_back( gp_Pnt( 137.685, 300, 0.449844) );
pnt4.push_back( gp_Pnt( 137.685, 300, -0.449844) );

pnt.push_back( gp_Pnt( 96, 0, 0.671985) );
pnt2.push_back( gp_Pnt( 96, 0, -0.671985) );
pnt3.push_back( gp_Pnt( 137.985, 300, 0.403191) );
pnt4.push_back( gp_Pnt( 137.985, 300, -0.403191) );

pnt.push_back( gp_Pnt( 96.5, 0, 0.592928) );
pnt2.push_back( gp_Pnt( 96.5, 0, -0.592928) );
pnt3.push_back( gp_Pnt( 138.285, 300, 0.355757) );
pnt4.push_back( gp_Pnt( 138.285, 300, -0.355757) );

pnt.push_back( gp_Pnt( 97, 0, 0.512533) );
pnt2.push_back( gp_Pnt( 97, 0, -0.512533) );
pnt3.push_back( gp_Pnt( 138.585, 300, 0.30752) );
pnt4.push_back( gp_Pnt( 138.585, 300, -0.30752) );

pnt.push_back( gp_Pnt( 97.5, 0, 0.430764) );
pnt2.push_back( gp_Pnt( 97.5, 0, -0.430764) );
pnt3.push_back( gp_Pnt( 138.885, 300, 0.258459) );
pnt4.push_back( gp_Pnt( 138.885, 300, -0.258459) );

pnt.push_back( gp_Pnt( 98, 0, 0.347585) );
pnt2.push_back( gp_Pnt( 98, 0, -0.347585) );
pnt3.push_back( gp_Pnt( 139.185, 300, 0.208551) );
pnt4.push_back( gp_Pnt( 139.185, 300, -0.208551) );

pnt.push_back( gp_Pnt( 98.5, 0, 0.262958) );
pnt2.push_back( gp_Pnt( 98.5, 0, -0.262958) );
pnt3.push_back( gp_Pnt( 139.485, 300, 0.157775) );
pnt4.push_back( gp_Pnt( 139.485, 300, -0.157775) );

pnt.push_back( gp_Pnt( 99, 0, 0.176844) );
pnt2.push_back( gp_Pnt( 99, 0, -0.176844) );
pnt3.push_back( gp_Pnt( 139.785, 300, 0.106106) );
pnt4.push_back( gp_Pnt( 139.785, 300, -0.106106) );

pnt.push_back( gp_Pnt( 99.5, 0, 0.0892042) );
pnt2.push_back( gp_Pnt( 99.5, 0, -0.0892042) );
pnt3.push_back( gp_Pnt( 140.085, 300, 0.0535225) );
pnt4.push_back( gp_Pnt( 140.085, 300, -0.0535225) );

pnt.push_back( gp_Pnt( 100, 0, 0) );
pnt2.push_back( gp_Pnt( 100, 0, 0) );
pnt3.push_back( gp_Pnt( 140.385, 300, 0) );
pnt4.push_back( gp_Pnt( 140.385, 300, 0) );

// end of point push backs


    for(int i = 0; i<sampleLen; i++)    // we sample x values on [0,1] with sampleLen points then append them to samples arrays to create BSplineCurve
    {

        samples->SetValue(i+1, pnt[i]);
        samples2->SetValue(i+1, pnt2[i]);
        samples3->SetValue(i+1, pnt3[i]);
        samples4->SetValue(i+1, pnt4[i]);

    }


    // base-upper
    GeomAPI_Interpolate inter(samples, false, GeometryTolerance);
    inter.Perform();
    Handle(Geom_BSplineCurve) spline = inter.Curve();
    TopoDS_Edge edge = BRepBuilderAPI_MakeEdge(spline);
    // base-lower
    GeomAPI_Interpolate inter2(samples2, false, GeometryTolerance);
    inter2.Perform();
    Handle(Geom_BSplineCurve) spline2 = inter2.Curve();
    TopoDS_Edge edge2 = BRepBuilderAPI_MakeEdge(spline2);

    // tip-upper
    GeomAPI_Interpolate inter3(samples3, false, GeometryTolerance);
    inter3.Perform();
    Handle(Geom_BSplineCurve) spline3 = inter3.Curve();
    TopoDS_Edge edge3 = BRepBuilderAPI_MakeEdge(spline3);
    // tip-lower
    GeomAPI_Interpolate inter4(samples4, false, GeometryTolerance);
    inter4.Perform();
    Handle(Geom_BSplineCurve) spline4 = inter4.Curve();
    TopoDS_Edge edge4 = BRepBuilderAPI_MakeEdge(spline4);

    TopTools_ListOfShape base, tip;
    base.Append(edge);
    base.Append(edge2);
    tip.Append(edge3);
    tip.Append(edge4);


    TopoDS_Wire baseWire, tipWire;
    TopoDS_Shape baseShape, tipShape;
    
    {
        BRepBuilderAPI_MakeWire wireMaker;
        wireMaker.Add(base);

        if(!wireMaker.IsDone())
        {
            cout << "baseWire not done!" << endl;
            return;
        }

        baseShape = wireMaker.Wire();

    }
    {
        BRepBuilderAPI_MakeWire wireMaker;
        wireMaker.Add(tip);

        if(!wireMaker.IsDone())
        {
            cout << "tipWire not done!" << endl;
            return;
        }

        tipShape = wireMaker.Wire();

    }

    BRepOffsetAPI_ThruSections lofter(false, false, 0.5);
    lofter.AddWire(TopoDS::Wire( baseShape)); 
    lofter.AddWire(TopoDS::Wire(tipShape));
    lofter.CheckCompatibility();
    lofter.Build();

    TopoDS_Shape result = lofter.Shape();

	mview->getContext() -> Remove(aisShape);
    // create an interactive shape from topological shape
    aisShape = new AIS_Shape(result);
	
    aisShape -> SetColor(Quantity_NOC_BLUE1);

    // define point attributes: define only the user defined marker point
    Handle(Prs3d_PointAspect) pa = new Prs3d_PointAspect(Aspect_TOM_PLUS, Quantity_NOC_BLUE1, 1);

    // apply point attributes to the interactive shape
    aisShape -> Attributes() -> SetPointAspect(pa);

	float lineWidth = 3.0f; Aspect_TypeOfLine lineType = Aspect_TOL_SOLID;

    // set the line width in reconstructed compound shape
    aisShape -> SetWidth(lineWidth);

    // set the line type for lines and wires of interactive shape
    // this could, for example, be solid, dotted or made up of dashes.
    aisShape -> Attributes() -> LineAspect() -> SetTypeOfLine(lineType);
    aisShape -> Attributes() -> WireAspect() -> SetTypeOfLine(lineType);



    // set the display mode of seen interactive object
    // if updateviewer equals Standard_False, the presentation of the interactive object returns
    // to the default selection mode; the object is displayed but no viewer will be updated
    mview->getContext() -> SetDisplayMode(aisShape, AIS_WireFrame, Standard_False);

    // controls the choice between the using the display and selection modes of open
    // local context which you have defined and activating those available by default
    mview->getContext() -> Display(aisShape);

    // if the vertices of the preview shape are not detectable
//    if (!detectVertex && shape.ShapeType() != TopAbs_VERTEX)
//    {

            // deactivates all the activated selection modes of interactive object
    // DO NOT ACTIVATE PREVIEW SHAPE, IT CAUSES SEGMENTATION FAULT ON NEWER OPENCASCADE VERSIONS (SOMETHING LATER THAN 6.5.1)
    mview->getContext() -> Deactivate(aisShape);

    mview->getView()->FitAll();




}




MainWindow::~MainWindow()
{
    delete ui;
}
